@@ -51,12 +51,13 @@
 #include <unistd.h>
 #include <pwd.h>
 
+#include <fstream>
 #include <vector>
 #include <string>
 #include <sstream>
 
-#include "hsa.h"
-#include "hsa_ext_amd.h"
+#include "hsa/hsa.h"
+#include "hsa/hsa_ext_amd.h"
 
 #define COL_BLU  "\x1B[34m"
 #define COL_KCYN  "\x1B[36m"
@@ -93,6 +94,8 @@ struct system_info_t {
     uint64_t max_wait = 0;
     hsa_endianness_t endianness;
     hsa_machine_model_t machine_model;
+    bool mwaitx_enabled;
+    bool dmabuf_support;
 };
 
 // This structure holds agent information acquired through hsa info related
@@ -113,6 +116,7 @@ struct agent_info_t {
   hsa_device_type_t device_type;
   uint32_t cache_size[4];
   uint32_t chip_id;
+  uint32_t asic_revision;
   uint32_t cacheline_size;
   uint32_t max_clock_freq;
   uint32_t internal_node_id;
@@ -133,6 +137,9 @@ struct agent_info_t {
   uint16_t workgroup_max_dim[3];
   uint16_t bdf_id;
   bool fast_f16;
+  uint32_t pkt_processor_ucode_ver;
+  uint32_t sdma_ucode_ver;
+  hsa_amd_iommu_version_t iommu_support;
 };
 
 // This structure holds memory pool information acquired through hsa info
@@ -271,6 +278,14 @@ static hsa_status_t AcquireSystemInfo(system_info_t *sys_info) {
   err = hsa_system_get_info(HSA_SYSTEM_INFO_MACHINE_MODEL,
                                                      &sys_info->machine_model);
   RET_IF_HSA_ERR(err);
+
+  // Get mwaitx mode
+  err = hsa_system_get_info(HSA_AMD_SYSTEM_INFO_MWAITX_ENABLED,
+                                                     &sys_info->mwaitx_enabled);
+  // Get DMABuf support
+  err = hsa_system_get_info(HSA_AMD_SYSTEM_INFO_DMABUF_SUPPORTED,
+                                                     &sys_info->dmabuf_support);
+  RET_IF_HSA_ERR(err);
   return err;
 }
 
@@ -296,6 +311,13 @@ static void DisplaySystemInfo(system_info_t const *sys_info) {
   } else if (HSA_ENDIANNESS_BIG == sys_info->endianness) {
     printValueStr("BIG");
   }
+
+  printLabel("Mwaitx:");
+  printf("%s\n", sys_info->mwaitx_enabled ? "ENABLED" : "DISABLED");
+
+  printLabel("DMAbuf Support:");
+  printf("%s\n", sys_info->dmabuf_support ? "YES" : "NO");
+
   printf("\n");
 }
 
@@ -382,6 +404,12 @@ AcquireAgentInfo(hsa_agent_t agent, agent_info_t *agent_i) {
                                                            &agent_i->chip_id);
   RET_IF_HSA_ERR(err);
 
+  // Get asic revision
+  err = hsa_agent_get_info(agent,
+                           (hsa_agent_info_t) HSA_AMD_AGENT_INFO_ASIC_REVISION,
+                                                           &agent_i->asic_revision);
+  RET_IF_HSA_ERR(err);
+
   // Get cacheline size
   err = hsa_agent_get_info(agent,
                        (hsa_agent_info_t) HSA_AMD_AGENT_INFO_CACHELINE_SIZE,
@@ -484,6 +512,19 @@ AcquireAgentInfo(hsa_agent_t agent, agent_info_t *agent_i) {
                     (hsa_agent_info_t)HSA_AMD_AGENT_INFO_MAX_WAVES_PER_CU,
                                                   &agent_i->max_waves_per_cu);
     RET_IF_HSA_ERR(err);
+
+    err = hsa_agent_get_info(agent,
+                    (hsa_agent_info_t)HSA_AMD_AGENT_INFO_UCODE_VERSION,
+                                                  &agent_i->pkt_processor_ucode_ver);
+    RET_IF_HSA_ERR(err);
+    err = hsa_agent_get_info(agent,
+                    (hsa_agent_info_t)HSA_AMD_AGENT_INFO_SDMA_UCODE_VERSION,
+                                                  &agent_i->sdma_ucode_ver);
+    RET_IF_HSA_ERR(err);
+    err = hsa_agent_get_info(agent,
+                    (hsa_agent_info_t)HSA_AMD_AGENT_INFO_IOMMU_SUPPORT,
+                                                  &agent_i->iommu_support);
+    RET_IF_HSA_ERR(err);
   }
   return err;
 }
@@ -567,6 +608,7 @@ static void DisplayAgentInfo(agent_info_t *agent_i) {
   }
 
   printLabelStr("Chip ID:", int_to_string(agent_i->chip_id), 1);
+  printLabelStr("ASIC Revision:", int_to_string(agent_i->asic_revision), 1);
   printLabelStr("Cacheline Size:", int_to_string(agent_i->cacheline_size), 1);
   printLabelInt("Max Clock Freq. (MHz):", agent_i->max_clock_freq, 1);
   printLabelInt("BDFID:", agent_i->bdf_id, 1);
@@ -618,6 +660,11 @@ static void DisplayAgentInfo(agent_info_t *agent_i) {
     printLabelStr("z", int_to_string(agent_i->grid_max_dim.z), 2);
 
     printLabelInt("Max fbarriers/Workgrp:", agent_i->fbarrier_max_size, 1);
+
+    printLabelInt("Packet Processor uCode::", agent_i->pkt_processor_ucode_ver, 1);
+    printLabelInt("SDMA engine uCode::", agent_i->sdma_ucode_ver, 1);
+    printLabelStr("IOMMU Support::",
+                    agent_i->iommu_support == HSA_IOMMU_SUPPORT_V2 ? "V2" : "None", 1);
   }
 }
 
@@ -1031,16 +1078,33 @@ AcquireAndDisplayAgentInfo(hsa_agent_t agent, void* data) {
 
 int CheckInitialState(void) {
   // Check kernel module for ROCk is loaded
-  FILE *fd = popen("lsmod | grep amdgpu", "r");
-  char buf[16];
-  if (fread (buf, 1, sizeof (buf), fd) == 0) {
+
+  std::ifstream amdgpu_initstate("/sys/module/amdgpu/initstate");
+  if (amdgpu_initstate){
+    std::stringstream buffer;
+    buffer << amdgpu_initstate.rdbuf();
+    amdgpu_initstate.close();
+
+    std::string line;
+    bool is_live = false;
+    while (std::getline(buffer, line)){
+      if (line.find( "live" ) != std::string::npos){
+        is_live = true;
+        break;
+      }
+    }
+    if (is_live){
+      printf("%sROCk module is loaded%s\n", COL_WHT, COL_RESET);
+    } else {
+      printf("%sROCk module is NOT live, possibly no GPU devices%s\n",
+                                                          COL_RED, COL_RESET);
+      return -1;
+    }
+  } else {
     printf("%sROCk module is NOT loaded, possibly no GPU devices%s\n",
                                                           COL_RED, COL_RESET);
     return -1;
-  } else {
-    printf("%sROCk module is loaded%s\n", COL_WHT, COL_RESET);
   }
-  pclose(fd);
 
   // Check if user belongs to the group for /dev/kfd (e.g. "video" or
   // "render")
